/**
 * \file cwallpaper.c
	CWallpaper
 * \author Colin Jones
 * \date 5-30-07
 */
//CWallpaper - A frontend for Wallpaper setting programs
// V0.3
// Copyright (c) 2007 Colin Jones

//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the license or (at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not write to the Free Software
//Foundation, Inc. 59 Temple Place - Suite 330, Boston, MA
//02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "cwallpaper_gtk.h"



gchar filename[2048];
gchar program[65];
gchar scaleOption[30];
gchar tileOption[30];
gchar centerOption[30];
gboolean options;
int currentOption;
GtkWidget *window;
GtkWidget *image;
GtkWidget *button;
GtkWidget *topbox;
GtkWidget *horbox;
GtkWidget *selectBox;
GtkWidget *optionBox;
GtkWidget *scrollable;	
GtkWidget *fileDialog;
GtkTreeModel *bgList;
GtkTreeSelection *selection;
GtkTreeIter    iter;

void loadSettings(gchar *configFile)
{
		/** 
		* Set the default options, in case the user doesn't have a config.
		**/
		strcpy(program, "fbsetbg");		
		strcpy(centerOption, "-c");
		strcpy(tileOption, "-t");
		strcpy(scaleOption, "-a");
		currentOption = 0; 
		options = TRUE;
		g_print("Attempting to use the config file: %s.\n", configFile);
		/**
		* Options: 0 = scale, 1 = center, 2 = tile
		**/
		FILE *file;		
		file = fopen(configFile, "r");
		
		if (file == NULL)
		{
			g_print("Config file was not found, using internal defaults (Hope you have fbsetbg :D ).\n", configFile);
			return;
		}
		if (file != NULL)
		{
			char c;
			char string[LENGTH_OF_FILENAME];
			int i = 0;
			gboolean getProgram = FALSE;
			gboolean getScale = FALSE;
			gboolean getCenter = FALSE;
			gboolean getDefault = FALSE;
			gboolean getTile = FALSE;
			gboolean scanning = TRUE;
			while (scanning)
			{
				c = fgetc(file);			
				if (c == EOF)
				{
					scanning = FALSE;					
				}
				if (c == 10 || c == 32 || c == EOF)
				{
					string[i] = '\0';
					if (getProgram)
					{
						g_print("Program = %s\n", string);
						strcpy(program, string);
						getProgram = FALSE;
					}
					if (getScale)
					{
						g_print("Scale = %s\n", string);
						strcpy(scaleOption, string);
						getScale = FALSE;
					}
					if (getDefault)
					{
						if (strcmp("c", string) == 0)
						{
							currentOption = 1;
							g_print("Default = Center\n", string);
						}
						if (strcmp("s", string) == 0)
						{
							currentOption = 0;
							g_print("Default = Scale\n", string);
						}
						if (strcmp("t", string) == 0)
						{
							currentOption = 2;
							g_print("Default = Tile\n", string);
						}
						if (strcmp("n", string) == 0)
						{
							g_print("Default = None\n", string);
							g_print("Options disabled\n", string);
							currentOption = 3;
							options = FALSE;
						}						
						getDefault = FALSE;
					}
					if (getCenter)
					{
						g_print("Center = %s\n", string);
						strcpy(centerOption, string);
						getCenter = FALSE;
					}
					if (getTile)
					{
						g_print("Tile = %s\n", string);
						strcpy(tileOption, string);
						getTile = FALSE;
					}
				 	
						if (strcmp(string, "program:") == 0)
						{
							getProgram = TRUE;
						}
						if (strcmp(string, "scale:") == 0)
						{
							getScale = TRUE;
						}
						if (strcmp(string, "tile:") == 0)
						{
							getTile = TRUE;
						}
						if (strcmp(string, "center:") == 0)
						{
							getCenter = TRUE;
						}
						if (strcmp(string, "default:") == 0)
						{
							getDefault = TRUE;
						}
					
					
					
					i = 0;
					
				}
				
				else
				{
					string[i] = c;
					i ++;
				}
			}
			fclose(file);
		}
}

void buildList()
{
	/**
	 * Now we need to make the list of wallpapers.
	 **/
	GtkListStore *listdata;
	GtkTreeIter iter;
	GError        *error = NULL;
	
		listdata = gtk_list_store_new(1, G_TYPE_STRING);
		FILE *file;
		gchar dataName[500];
		strcpy(dataName, getenv("HOME"));	
		strcat(dataName, "/.config/cwallpaper/wallpapers");
		file = fopen(dataName, "r");
		if (file == NULL)
		{
			gtk_list_store_append(listdata, &iter);					
			gtk_list_store_set(listdata, &iter, COL_FILENAME, "You need to make a wallpaper list in:", -1);
			gtk_list_store_append(listdata, &iter);					
			gtk_list_store_set(listdata, &iter, COL_FILENAME, dataName, -1);
			gtk_list_store_append(listdata, &iter);					
			gtk_list_store_set(listdata, &iter, COL_FILENAME, " ", -1);
			gtk_list_store_append(listdata, &iter);					
			gtk_list_store_set(listdata, &iter, COL_FILENAME, "Just list the path to each wallpaper in the format:", -1);
			gtk_list_store_append(listdata, &iter);					
			gtk_list_store_set(listdata, &iter, COL_FILENAME, "/path/to/wallpaper/wallpaper.png", -1);
		}
		if (file != NULL)
		{
			char c;
			char string[LENGTH_OF_FILENAME];
			int i = 0;
			gboolean scanning = TRUE;
			while (scanning)
			{
				c = fgetc(file);
				if (c == EOF)
				{
					scanning = FALSE;					
				}
				if (c == 10 || c == EOF) //c =32
				{
						string[i] = '\0';
						gtk_list_store_append(listdata, &iter);
					
						gtk_list_store_set(listdata, &iter, COL_FILENAME, string, -1);					
					i = 0;
					
				}
				
				else
				{
					string[i] = c;
					i ++;
				}
			}
			fclose(file);
		}	
	bgList = GTK_TREE_MODEL(listdata);
}

void saveLast()
{
	printf("Saving current wallpaper.\n");
	FILE *file;
	gchar dataName[500];
	gchar lastWallpaperString[LENGTH_OF_FILENAME];
	strcpy(dataName, getenv("HOME"));	
	strcat(dataName, "/.config/cwallpaper/lastwallpaper");
	file = fopen(dataName, "w");
	
	if (options == FALSE)
	{
		strcpy(lastWallpaperString, "n: ");
	}
	else 
	{
		if (currentOption == 0)
		{
			//Scale
			strcpy(lastWallpaperString, "s: ");
		}
		if (currentOption == 1)
		{
			//Scale
			strcpy(lastWallpaperString, "c: ");
		}
		if (currentOption == 2)
		{
			//Scale
			strcpy(lastWallpaperString, "t: ");
		}
	}
	strcat(lastWallpaperString, filename);
	
	fputs(lastWallpaperString, file);
	fclose(file);
	
	
}

void loadAndSetLast()
{
	printf("Loading last wallpaper.\n");
	FILE *file;
	gchar dataName[500];
	strcpy(dataName, getenv("HOME"));	
	strcat(dataName, "/.config/cwallpaper/lastwallpaper");
	file = fopen(dataName, "r");
	
	if (file == NULL)
		{
			g_print("Last wallpaper not found. (You have not used CWallpaper before)\n");
			return;
		}
		if (file != NULL)
		{
			char c;
			char string[LENGTH_OF_FILENAME];
			int i = 0;
			gboolean getType = TRUE;

			gboolean scanning = TRUE;
			while (scanning)
			{
				c = fgetc(file);			
				if (c == EOF)
				{
					scanning = FALSE;					
				}
				if (c == 10 || c == 32 || c == EOF)
				{
					string[i] = '\0';
					if (getType)
					{
						if (strcmp("c:", string) == 0)
						{
							currentOption = 1;
							g_print("Default = Center\n", string);
						}
						if (strcmp("s:", string) == 0)
						{
							currentOption = 0;
							g_print("Default = Scale\n", string);
						}
						if (strcmp("t:", string) == 0)
						{
							currentOption = 2;
							g_print("Default = Tile\n", string);
						}
						if (strcmp("n:", string) == 0)
						{
							g_print("Default = None\n", string);
							g_print("Options disabled\n", string);
							currentOption = 3;
							options = FALSE;
						}						
						getType = FALSE;
					}
					else
					{					
						strcpy(filename, string);
					}
					
					
					i = 0;
					
				}
				
				else
				{
					string[i] = c;
					i ++;
				}
			}
			fclose(file);
			
		}
		setBG();
	
}

void createTreeList()
{
	GtkCellRenderer     *viewer;
	GtkTreeViewColumn *col;
	g_print("Loading\n");
	buildList();

	selectBox = gtk_tree_view_new_with_model(bgList);
	
selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selectBox));
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selectImage), NULL);
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Wallpaper");
	
		
	viewer = gtk_cell_renderer_text_new();	
	
	gtk_tree_view_column_pack_start(col, viewer, FALSE);
	gtk_tree_view_column_set_attributes(col, viewer, "text", COL_FILENAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(selectBox), col);
	
	g_object_unref (bgList);
}


static void setOption(GtkWidget *widget, gpointer  data)
{	
	currentOption = gtk_combo_box_get_active(GTK_COMBO_BOX(optionBox));
}
static void setWallpaper( GtkWidget *widget, gpointer  data)
{
	if (gtk_tree_selection_get_selected(selection, &bgList, &iter))
	{
		gchar *name;

		gtk_tree_model_get (bgList, &iter, COL_FILENAME, &name, -1);
		strcpy(filename, "\"");
               strcat(filename, name);
               strcat(filename, "\"");
		g_print ("Setting the wallpaper to: %s\n", filename);
		
		setBG();
		
	}	
}

static void selectImage( GtkWidget *widget, gpointer  data)
{
      
	if (gtk_tree_selection_get_selected(selection, &bgList, &iter))
	{
		gchar *name;


		gtk_tree_model_get (bgList, &iter, COL_FILENAME, &name, -1);

		strcpy(filename, name);
		//g_print ("selected row is: %s\n", filename);
		
		
		loadAndDisplayImage(image);
	}
}
static gboolean delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
    g_print ("Quiting peaceablly\n");
	gtk_main_quit();
    return FALSE;
}

void makeGUI()
{	
	GtkWidget *seperator;
	GtkWidget* vertbox;
	/**
	* Create the window and the main containter for all the widgets (topbox)
	**/
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	topbox = gtk_vbox_new(FALSE, 0);
	
	
	
	
		
	/*
	* Build the left side of the panel first
	*/
	
	image = gtk_image_new();
	gtk_widget_set_size_request(image, 320, 240);
	horbox = gtk_hbox_new(FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(topbox), horbox, TRUE, TRUE, 0);
	
	
	vertbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(horbox), vertbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vertbox), image, TRUE, FALSE, 0);
	gtk_widget_show(vertbox);
	gtk_widget_show(image);	
	if (options)
	{
		GtkWidget* frame;
		frame = gtk_frame_new("Wallpaper Settings");		
		optionBox = gtk_combo_box_new_text ();  	
		gtk_combo_box_append_text (GTK_COMBO_BOX (optionBox), "Scale To Fit");
		gtk_combo_box_append_text (GTK_COMBO_BOX (optionBox), "Center");
		gtk_combo_box_append_text (GTK_COMBO_BOX (optionBox), "Tile");		
		gtk_combo_box_set_active (GTK_COMBO_BOX (optionBox), 0);	
		g_signal_connect (G_OBJECT (GTK_COMBO_BOX (optionBox)), "changed", G_CALLBACK (setOption), NULL);
		gtk_widget_show(optionBox);
		gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
		gtk_container_add (GTK_CONTAINER (frame), optionBox);
		gtk_widget_show(frame);		
		gtk_box_pack_start(GTK_BOX(vertbox), frame, TRUE, TRUE, 0);
		seperator = gtk_hseparator_new();		
		gtk_widget_show(seperator);
	}
	
	/**
	* Seperate the two sides, just for fun
	**/
	seperator = gtk_vseparator_new();
	gtk_widget_show(seperator);
	gtk_box_pack_start(GTK_BOX(horbox), seperator, TRUE, TRUE, 2);	
	
	/**
	* Now make the right side of the panel.
	* This is where the list of wallpapers go (selectbox)
	* It is embed in a scrollable area, so the window isn't 
	* twenty meters long
	**/	
	scrollable = gtk_scrolled_window_new (NULL, NULL);
	createTreeList();
	
	
	gtk_widget_set_size_request(scrollable, 250, 240);
	gtk_container_add(GTK_CONTAINER(scrollable), selectBox);
	
	
	
	
	
	gtk_box_pack_start(GTK_BOX(horbox), scrollable, TRUE, TRUE, 0);	
	gtk_widget_show(selectBox);
	gtk_widget_show(scrollable);
	gtk_widget_show(horbox);
	/*
	* Foo!
	*/
	horbox = gtk_hbox_new(TRUE, 0);
	

	
	
	seperator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(topbox), seperator, TRUE, TRUE, 2);
	gtk_widget_show(seperator);
	
	
	
	
	
	gtk_box_pack_start(GTK_BOX(topbox), horbox, FALSE, TRUE, 2);
	
	button = gtk_button_new_from_stock("gtk-apply");
	gtk_box_pack_start(GTK_BOX(horbox), button, FALSE, TRUE, 10);
	
	
	gtk_widget_show(button);
	gtk_widget_show(horbox);
	
	

	
	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(delete_event), NULL);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(setWallpaper), "2");
	button =  gtk_button_new_from_stock("gtk-cancel");   
	gtk_box_pack_end(GTK_BOX(horbox), button, FALSE, TRUE, 10);
	gtk_widget_show(button);
	g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (delete_event), G_OBJECT (window));
	
	gtk_container_add(GTK_CONTAINER(window), topbox);
	
	

	gtk_widget_show(topbox);
	 gtk_window_set_title (GTK_WINDOW (window), "CWallpaper Setter");
	gtk_widget_show(window);
	gtk_widget_grab_focus(GTK_WIDGET(selectBox));
	buildList();
	
	return;
}



void loadAndDisplayImage(GtkWidget *targetImage)
{
	GdkPixbuf *pixImage;
	pixImage = gdk_pixbuf_new_from_file_at_size(filename, 320, 200, NULL);
	gtk_image_set_from_pixbuf(GTK_IMAGE(targetImage), pixImage);
	g_object_unref(pixImage);
}

void setBG()
{
	

	gchar command[2500];
	//g_print("Using the program %s.\n", program);
	strcpy(command, "");
	strcpy(command, program);
	strcat(command, " ");
	switch (currentOption)
	{
		case 0:
			strcat(command, scaleOption);
			break;
		case 1:
			strcat(command, centerOption);
			break;
		case 2:
			strcat(command, tileOption);
			break;
		case 3:
			g_print("No options.\n");
			break;
	}
	strcat(command, " ");
	if (filename[0] != '\0')
	{
		strcat(command, filename);
		system(command);
		saveLast();
	}
	return;
}

int main(int argv, char *argc[])
{
	gchar dataName[500];
	strcpy(dataName, getenv("HOME"));		
	strcat(dataName, "/.config/cwallpaper/config");
	int configIndex = 0;
	gboolean lastWallpaper = FALSE;
	if (argv < 2)
	{
		loadSettings(dataName);	
	}
	else //Open the file which is arg 1 
	{
		int x;
		for (x =0; x < argv; x++)
		{
			//Check for additional options.
			if (strcmp(argc[x], "-l") == 0)
			{
				lastWallpaper = TRUE;
				
			}
			if (strcmp(argc[x], "-c") == 0)
			{
				configIndex = x + 1;
				
			}
		}
		if (configIndex > 0)
		{
			loadSettings(argc[configIndex]);
		}
		else
		{
			loadSettings(dataName);
		}
		if (lastWallpaper == TRUE)
		{
			loadAndSetLast();
			return 0;
		}
	}
	
	gtk_init(&argv, &argc);	
	makeGUI();
	
	
	gtk_main();
	return 0;
}
